package com.example.jason.aicompetition;

import android.Manifest;
import android.app.Activity;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.content.res.AssetManager;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.hardware.Camera;
import android.os.Handler;
import android.os.Message;
import android.os.Bundle;
import android.support.v4.app.ActivityCompat;
import android.util.Log;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.Toast;

import com.example.jason.aicompetition.services.SpeechServices;
import com.example.jason.aicompetition.util.CameraPreview;
import com.example.jason.aicompetition.util.DetectedObject;
import com.example.jason.aicompetition.util.MobilenetSsd;
import com.example.jason.aicompetition.util.PhotoManager;

import java.io.BufferedReader;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;

import static android.os.SystemClock.sleep;
import static com.example.jason.aicompetition.application.SpeechApp.IMAGE_URL;

public class MainActivity extends Activity {
    private final String TAG = "MainActivity";
    private final int REQUEST_WRITE_EXTERNAL_STORAGE = 1;
    public static final int MSG_FACE_1 = 1;
    public static final int MSG_FACE_2 = 2;
    public static final int MSG_FACE_3 = 3;
    public static final int MSG_FACE_4 = 4;
    public static final int MSG_SET_IMAGE = 5;
    public static final int MSG_DISMISS_CAMERA = 6;
    public static final int MSG_VISIBLE_CAMERA = 7;

    private ArrayList<String> mResultLabel = new ArrayList<>();
    private ArrayList<String> mResultLabelCn = new ArrayList<>();
    private MobilenetSsd mMobilenetSsd = new MobilenetSsd();
    private ArrayList<DetectedObject> mResultList = new ArrayList<>();
    private static SpeechServices mSpeechServices;

    private CameraPreview mPreview;
    PhotoManager mPhotoManager;

    private boolean mLoadResult = false;
    private final int[] ddims = {1, 3, 300, 300};

    private ImageView imageView;
    FrameLayout preview;
    private Bitmap bmp = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Log.d(TAG, "onCreate");
        setContentView(R.layout.demo_with_face);
        imageView = (ImageView) findViewById(R.id.sleep_face);
        preview = (FrameLayout) findViewById(R.id.photo_preview);

        mPhotoManager = PhotoManager.getInstance();
        mPhotoManager.init(this, mPicture, mHandler);

        mPreview = new CameraPreview(this, mPhotoManager.getCameraInstance());
        preview.addView(mPreview);
        preview.setVisibility(View.GONE);

        SpeechServices.setHandler(mHandler);


        //读写权限
        checkPermission();

        Intent i = new Intent(this, SpeechServices.class);
        startService(i);

        try {
            initMobilenetSsd();
        } catch (IOException e) {
            Log.e(TAG, "initMobilenetSsd error");
        }

        readCacheLabelFromLocalFile();
    }

    private Camera.PictureCallback mPicture = new Camera.PictureCallback() {
        @Override
        public void onPictureTaken(byte[] data, Camera camera) {

            // 获取Jpeg图片，并保存在sd卡上，拍照成功回调函数
            File pictureFile = new File(IMAGE_URL);

            try {
                pictureFile.createNewFile();
            }
            catch (Exception e){
                Log.w(TAG, "onPictureTaken: pictureFile create fail " + e);
            }

            try {
                FileOutputStream fos = new FileOutputStream(pictureFile);
                fos.write(data);
                fos.close();
                //分析照片
                new Thread(new Runnable() {
                    @Override
                    public void run() {
                        predictImage(IMAGE_URL);
                    }
                }).start();

                Log.d(TAG, "onPictureTaken: take picture successfully");
            } catch (Exception e) {
                Log.d(TAG, "保存图片失败" + e);
            }

        }
    };

    //检查读写权限
    private void checkPermission() {
        //检查权限（NEED_PERMISSION）是否被授权 PackageManager.PERMISSION_GRANTED表示同意授权
        if (ActivityCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE)
                != PackageManager.PERMISSION_GRANTED) {
            //用户已经拒绝过一次，再次弹出权限申请对话框需要给用户一个解释
            if (ActivityCompat.shouldShowRequestPermissionRationale(this, Manifest.permission
                    .WRITE_EXTERNAL_STORAGE)) {
                Toast.makeText(this, "请开通储存权限，否则会耗费更多流量！", Toast.LENGTH_SHORT).show();
            }
            //申请权限
            ActivityCompat.requestPermissions(this, new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE},
                    REQUEST_WRITE_EXTERNAL_STORAGE);

        } else {
            //Toast.makeText(this, "授权成功！", Toast.LENGTH_SHORT).show();
            Log.i(TAG, "checkPermission: 已经授权！write");
        }
    }

    //读取模型的种类
    private void readCacheLabelFromLocalFile() {
        try {
            AssetManager assetManager = getApplicationContext().getAssets();
            BufferedReader reader = new BufferedReader(new InputStreamReader(assetManager.open("word_mobile.txt")));
            String readLine = null;
            while ((readLine = reader.readLine()) != null) {
                Log.i(TAG, "readCacheLabelFromLocalFile: " + readLine);
                String [] temp = readLine.split(" ");
                if (temp.length == 2) {
                    mResultLabel.add(temp[0]);
                    mResultLabelCn.add(temp[1]);
                }
                else {
                    Log.w(TAG, "readCacheLabelFromLocalFile: word_mobile.txt format error");
                }
            }
            reader.close();
        } catch (Exception e) {
            Log.e(TAG, "error " + e);
        }
    }

    //初始化模型
    private void initMobilenetSsd() throws IOException {
        byte[] param = null;
        byte[] bin = null;
        {
            //InputStream assetsInputStream = getAssets().open("MobilenetSsd-tiny_tarmac.param.bin");
            InputStream assetsInputStream = getAssets().open("mobilenet_ssd.param.bin");
            int available = assetsInputStream.available();
            param = new byte[available];
            int byteCode = assetsInputStream.read(param);
            assetsInputStream.close();
        }
        {
            //InputStream assetsInputStream = getAssets().open("MobilenetSsd-tiny_tarmac1.bin");
            InputStream assetsInputStream = getAssets().open("mobilenet_ssd.bin");
            int available = assetsInputStream.available();
            bin = new byte[available];
            int byteCode = assetsInputStream.read(bin);
            assetsInputStream.close();
        }

        mLoadResult = mMobilenetSsd.Init(param, bin);
        Log.d(TAG, "yolov2tiny_load_model_result:" + mLoadResult);
    }

    //  predict image
    private void predictImage(String image_path) {
        // picture to float array
        Bitmap bmp = mPhotoManager.getScaleBitmap(image_path);
        Bitmap rgba = bmp.copy(Bitmap.Config.ARGB_8888, true);

        // resize to 300x300
        Bitmap input_bmp = Bitmap.createScaledBitmap(rgba, ddims[2], ddims[3], false);
        try {
            // Data format conversion takes too long
            long start = System.currentTimeMillis();
            // get predict result
            detect(input_bmp);
            Log.d(TAG, "length of result: " + mResultList.size());

            long end = System.currentTimeMillis();
            //Log.d(TAG, "origin predict result:" + Arrays.toString(result));
            long time = end - start;
            // show predict result and time
            //float[] r = get_max_result(result);

            StringBuilder showText = new StringBuilder();
            DetectedObject bottle = null;
            Canvas canvas = new Canvas(rgba);
            //图像上画矩形
            for (DetectedObject object : mResultList) {
                //画矩形
                Paint paint = new Paint();
                paint.setColor(Color.RED);
                paint.setStyle(Paint.Style.STROKE);//不填充
                paint.setStrokeWidth(2); //线的宽度
                canvas.drawRect(object.getLeft() * rgba.getWidth(), object.getTop() * rgba.getHeight(),
                        object.getRight() * rgba.getWidth(), object.getBottom() * rgba.getHeight(), paint);

                showText.append("result\nname：");
                showText.append(object.getName());
                showText.append("\tprobability：");
                showText.append(object.getProbability());
                showText.append("\n");

                //判断是否有关键字：杯子
                if (object.getName().equals("bottle")){
                    bottle = object;
                }
            }
            showText.append("time: (ms) ");
            showText.append(time);

            Log.i(TAG, "predictImage: " + showText.toString());
            if(bottle != null){
                //识别物体有杯子
                mResultList.remove(bottle);
                //获得位置语音的文字
                String ans = bottle.localization(mResultList);
                mSpeechServices.startSpeaking(ans);
                Log.i(TAG, "predictImage: localization" + ans);
            }
            else {
                mSpeechServices.startSpeaking("我没有看到杯子");
            }
            //show_image.setImageBitmap(rgba);
            //关闭预览
            Message message = mHandler.obtainMessage(MSG_DISMISS_CAMERA);
            Bundle bundle = new Bundle();
            bundle.putParcelable("bitmap", rgba);
            message.setData(bundle);
            mHandler.sendMessageDelayed(message, 2000);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    //物体识别
    private void detect(Bitmap input_bmp) {
        mResultList.clear();
        //探测物体，返回结果result
        float[] result = mMobilenetSsd.Detect(input_bmp);
        //Log.i(TAG, "detect: result num: " + result.length / 6);
        int objectSize = result.length;
        Log.i(TAG, "detect: result num: " + objectSize / 6);
        for (int i = 0; i < objectSize; ) {
            int id = (int) result[i++];
            String name = mResultLabel.get(id);
            String nameCn = mResultLabelCn.get(id);
            float pro = result[i++];
            float left = result[i++];
            float top = result[i++];
            float right = result[i++];
            float bottom = result[i++];
            DetectedObject object = new DetectedObject(name, nameCn, id, pro, left, top, right, bottom);
            mResultList.add(object);
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        mResultLabel = null;
        mMobilenetSsd = null;

        mPhotoManager.clear();
        mPhotoManager = null;
        Log.d(TAG, "onDestroy");
    }

    public static void registerService(SpeechServices speechServices){
        mSpeechServices = speechServices;
    }

    public static void unRegisterService(){
        mSpeechServices = null;
    }

    private Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case MSG_FACE_1:
                    imageView.setImageResource(R.drawable.face1);
                    break;
                case MSG_FACE_2:
                    imageView.setImageResource(R.drawable.face2);
                    break;
                case MSG_FACE_3:
                    imageView.setImageResource(R.drawable.face3);
                    break;
                case MSG_FACE_4:
                    imageView.setImageResource(R.drawable.face4);
                    break;
                case MSG_SET_IMAGE:
                    Bundle bundle = msg.getData();
                    byte[] bmap = bundle.getByteArray("bitmap");
                    ByteArrayOutputStream outPut = new ByteArrayOutputStream();
                    bmp = BitmapFactory.decodeByteArray(bmap, 0, bmap.length);
                    bmp.compress(Bitmap.CompressFormat.PNG, 100, outPut);
                    imageView.setImageBitmap(bmp);
                case MSG_DISMISS_CAMERA:
                    //关闭预览
                    mPhotoManager.stopPreview();
                    preview.setVisibility(View.GONE);
                    //展示结果
                    bmp = (Bitmap) msg.getData().getParcelable("bitmap");
                    imageView.setImageBitmap(bmp);
                    Message message = mHandler.obtainMessage(MSG_FACE_2);
                    mHandler.sendMessageDelayed(message, 2000);
                    break;
                case MSG_VISIBLE_CAMERA:
                    preview.setVisibility(View.VISIBLE);
                    mPhotoManager.startPreview();
                    break;
                default:
                    break;

            }
        }
    };
}
