package com.example.jason.aicompetition.util;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.hardware.Camera;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

import static android.os.SystemClock.sleep;
import static com.example.jason.aicompetition.MainActivity.MSG_VISIBLE_CAMERA;

public class PhotoManager {

    private static PhotoManager instance = new PhotoManager();
    private final String TAG = "PhotoManager";

    private Context mContext;
    private Handler mHandler;
    private Camera mCamera;
    private Camera.PictureCallback mPicture;

    public static PhotoManager getInstance() {
        return instance;
    }

    public void init(Context context, Camera.PictureCallback picture, Handler handler) {
        mContext = context;
        mPicture = picture;
        mHandler = handler;
    }

    // compress picture
    public Bitmap getScaleBitmap(String filePath) {
        BitmapFactory.Options opt = new BitmapFactory.Options();
        opt.inJustDecodeBounds = true;
        BitmapFactory.decodeFile(filePath, opt);

        int bmpWidth = opt.outWidth;
        int bmpHeight = opt.outHeight;

        int maxSize = 500;

        // compress picture with inSampleSize
        opt.inSampleSize = 1;
        while (true) {
            if (bmpWidth / opt.inSampleSize < maxSize || bmpHeight / opt.inSampleSize < maxSize) {
                break;
            }
            opt.inSampleSize *= 2;
        }
        opt.inJustDecodeBounds = false;
        return BitmapFactory.decodeFile(filePath, opt);
    }

    //打开一个Camera
    public Camera getCameraInstance() {
        if (mCamera == null){
            try {
                mCamera = Camera.open(Camera.CameraInfo.CAMERA_FACING_BACK);
                Log.d(TAG, "打开一个Camera");
            } catch (Exception e) {
                Log.d(TAG, "打开Camera失败");
            }
        }
        return mCamera;
    }

    //拍照
    public void takePicture() {
        //打开预览
        Message message = mHandler.obtainMessage(MSG_VISIBLE_CAMERA);
        mHandler.sendMessage(message);
        new Thread(new Runnable() {
            @Override
            public void run() {
                sleep(1000);
                //拍照
                if(mCamera != null) {
                    mCamera.takePicture(null, null, mPicture);
                }
            }
        }).start();
        Log.d(TAG, "takePicture");
    }

    //开启预览
    public void startPreview(){
        if (mCamera != null){
            mCamera.startPreview();
        }
    }

    //关闭预览
    public void stopPreview(){
        mCamera.stopPreview();
    }

    public void clear() {
        if (mCamera != null) {
            mCamera.release();
            mCamera = null;
        }
        mContext = null;
        mPicture = null;
        mHandler = null;
        Log.d(TAG, "clear");
    }
}

